@@ -25,6 +25,7 @@ class GreedyAgent(BaseAgent):
     def __init__(self, action_space):
         BaseAgent.__init__(self, action_space)
         self.tested_action = None
+        self.resulting_rewards = None
 
     def act(self, observation, reward, done=False):
         """
@@ -53,11 +54,11 @@ class GreedyAgent(BaseAgent):
         """
         self.tested_action = self._get_tested_action(observation)
         if len(self.tested_action) > 1:
-            all_rewards = np.full(shape=len(self.tested_action), fill_value=np.NaN, dtype=dt_float)
+            self.resulting_rewards = np.full(shape=len(self.tested_action), fill_value=np.NaN, dtype=dt_float)
             for i, action in enumerate(self.tested_action):
                 simul_obs, simul_reward, simul_has_error, simul_info = observation.simulate(action)
-                all_rewards[i] = simul_reward
-            reward_idx = int(np.argmax(all_rewards))  # rewards.index(max(rewards))
+                self.resulting_rewards[i] = simul_reward
+            reward_idx = int(np.argmax(self.resulting_rewards))  # rewards.index(max(rewards))
             best_action = self.tested_action[reward_idx]
         else:
             best_action = self.tested_action[0]
