@@ -3,6 +3,7 @@ __all__ = [
     "GridValue",
     "ChangeNothing",
     "MultiFolder",
+    "MultifolderWithCache",
     "GridStateFromFile",
     "GridStateFromFileWithForecasts",
     "GridStateFromFileWithForecastsWithMaintenance",
@@ -17,3 +18,4 @@ from grid2op.Chronics.GridStateFromFileWithForecasts import GridStateFromFileWit
 from grid2op.Chronics.MultiFolder import Multifolder
 from grid2op.Chronics.ReadPypowNetData import ReadPypowNetData
 from grid2op.Chronics.GSFFWFWM import GridStateFromFileWithForecastsWithMaintenance
+from grid2op.Chronics.MultifolderWithCache import MultifolderWithCache
