@@ -3,7 +3,7 @@ __all__ = [
     "ToVect",
     "IdToAct",
     "ConnectivityConverter",
-    "AnalogStateConverter",
+    "AnalogStateConverter"
 ]
 
 from grid2op.Converter.Converters import Converter
@@ -11,4 +11,3 @@ from grid2op.Converter.ToVect import ToVect
 from grid2op.Converter.IdToAct import IdToAct
 from grid2op.Converter.AnalogStateConverter import AnalogStateConverter
 from grid2op.Converter.ConnectivityConverter import ConnectivityConverter
-
