@@ -8,7 +8,7 @@
 
 import numpy as np
 from grid2op.Reward.BaseReward import BaseReward
-
+from grid2op.dtypes import dt_float
 
 class IncreasingFlatReward(BaseReward):
     """
@@ -19,9 +19,9 @@ class IncreasingFlatReward(BaseReward):
     """
     def __init__(self, per_timestep=1):
         BaseReward.__init__(self)
-        self.per_timestep = per_timestep
-        self.total_reward = 0
-        self.reward_min = 0
+        self.per_timestep = dt_float(per_timestep)
+        self.total_reward = dt_float(0.0)
+        self.reward_min = dt_float(0.0)
 
     def initialize(self, env):
         if env.chronics_handler.max_timestep() > 0:
@@ -31,7 +31,7 @@ class IncreasingFlatReward(BaseReward):
 
     def __call__(self, action, env, has_error, is_done, is_illegal, is_ambiguous):
         if not has_error:
-            res = env.nb_time_step * self.per_timestep
+            res = dt_float(env.nb_time_step * self.per_timestep)
         else:
             res = self.reward_min
         return res
