@@ -15,7 +15,7 @@ class DefaultRules(LookParam, PreventReconnection):
     This subclass combine both :class:`LookParam` and :class:`PreventReconnection`.
     An action is declared legal if and only if:
 
-      - It doesn't diconnect / reconnect more power lines than  what stated in the actual game _parameters
+      - It doesn't disconnect / reconnect more power lines than  what stated in the actual game _parameters
         :class:`grid2op.Parameters`
       - It doesn't attempt to act on more substations that what is stated in the actual game _parameters
         :class:`grid2op.Parameters`
